==> NVIDIA-FreeBSD-x86-100.14.09.tar.gz: NVIDIA-FreeBSD-x86-100.14.09/src/nvidia_ctl.c <==
/* _NVRM_COPYRIGHT_BEGIN_
 *
 * Copyright 2001-2002 by NVIDIA Corporation.  All rights reserved.  All
 * information contained herein is proprietary and confidential to NVIDIA
 * Corporation.  Any use, reproduction, or disclosure without the written
 * permission of NVIDIA Corporation is prohibited.
 *
 * _NVRM_COPYRIGHT_END_
 */

#include "nv-misc.h"
#include "os-interface.h"
#include "nv.h"
#include "nv-freebsd.h"

static d_open_t  nvidia_ctl_open;
static d_close_t nvidia_ctl_close;
static d_ioctl_t nvidia_ctl_ioctl;
static d_poll_t  nvidia_ctl_poll;

static struct cdevsw nvidia_ctl_cdevsw = {
    .d_open =      nvidia_ctl_open,
    .d_close =     nvidia_ctl_close,
    .d_ioctl =     nvidia_ctl_ioctl,
    .d_poll =      nvidia_ctl_poll,
    .d_name =      "nvidiactl",
    .d_version =   D_VERSION,
    .d_flags =     D_TRACKCLOSE|D_NEEDGIANT
};

static struct cdev *nvidia_ctl_cdev = NULL;
struct nvidia_softc nvidia_ctl_sc;

static int nvidia_count = 0;

int nvidia_ctl_open(
    struct cdev *dev,
    int oflags,
    int devtype,
    d_thread_t *td
)
{
    int status;
    nv_state_t *nv = &nvidia_ctl_state;

    nv_lock_api(nv);
    status = nvidia_open_ctl(dev, td);
    nv_unlock_api(nv);

    return status;
}

int nvidia_ctl_close(
    struct cdev *dev,
    int fflag,
    int devtype,
    d_thread_t *td
)
{
    int status;
    nv_state_t *nv = &nvidia_ctl_state;

    nv_lock_api(nv);
    status = nvidia_close_ctl(dev, td);
    nv_unlock_api(nv);

    return status;
}

int nvidia_ctl_ioctl(
    struct cdev *dev,
    u_long cmd,
    caddr_t data,
    int fflag,
    d_thread_t *td
)
{
    int status = 0;
    nv_state_t *nv = &nvidia_ctl_state;

    if (__NV_IOC_TYPE(cmd) != NV_IOCTL_MAGIC)
        return ENOTTY;

    nv_lock_api(nv);

    switch (__NV_IOC_NR(cmd)) {
        case NV_ESC_CARD_INFO:
            do {
                struct nv_ioctl_card_info *ci = (void *) data;
                status = nvidia_get_card_info(ci);
            } while (0);
            break;

        case NV_ESC_POST_VBIOS:
            do {
                struct nv_ioctl_post_vbios *pc = (void *) data;
                status = nvidia_post_vbios(pc);
            } while (0);
            break;

        case NV_ESC_CHECK_VERSION_STR:
            do {
                nv_ioctl_rm_api_version_t *params = (void *) data;
                status = (rm_perform_version_check(NULL, params) == RM_OK) ? 0 : -EINVAL;
            } while (0);
            break;

        default:
            status = nvidia_handle_ioctl(dev, cmd, data, fflag, td);
    }

    nv_unlock_api(nv);
    return status;
}


/*
 * The X driver module adds the ctrl device fd to the XFree86 select list
 * and receives notification if the select status changes. We make use of
 * this mechanism to signal hotkey events; this function is called by the
 * core resource manager whenever a legacy hotkey event is detected.
 */

void nv_set_hotkey_occurred_flag(void)
{
    nv_state_t *nv;
    struct nvidia_softc *sc;

    nv = &nvidia_ctl_state;
    sc = nv->os_state;

    nv_lock_rm(nv);
    nv->flags |= NV_FLAG_HOTKEY_OCCURRED;
    nv_unlock_rm(nv);

    selwakeup(&sc->rsel);
}

int nvidia_ctl_poll(
    struct cdev *dev,
    int events,
    d_thread_t *td
)
{
    nv_state_t *nv;
    struct nvidia_softc *sc;

    nv = &nvidia_ctl_state;
    sc = nv->os_state;

    nv_lock_rm(nv);

    if ((nv->flags & NV_FLAG_HOTKEY_OCCURRED) == 0) {
        nv_unlock_rm(nv);
        selrecord(td, &sc->rsel);
    } else {
        nv->flags &= ~NV_FLAG_HOTKEY_OCCURRED;
        nv_unlock_rm(nv);

        return (events & (POLLIN | POLLRDNORM)); /* ? */
    }

    return 0;
}

int nvidia_ctl_attach(void)
{
    if (nvidia_count == 0) {
        /*
         * This routine is called from nvidia_attach(), multiple times
         * when more than one device is installed.
         */
        nvidia_ctl_cdev = make_dev(&nvidia_ctl_cdevsw,
                CDEV_CTL_MINOR,
                UID_ROOT, GID_WHEEL, 0666,
                "%s", nvidia_ctl_cdevsw.d_name);
    }

    nvidia_count++;
    return 0;
}

int nvidia_ctl_detach(void)
{
    struct nvidia_softc *sc;

    sc = &nvidia_ctl_sc;
    nvidia_count--;

    if (nvidia_count == 0) {
        /*
         * Like nvidia_ctl_attach(), nvidia_ctl_detach() will also be
         * called more than once with multiple devices.
         */
        destroy_dev(nvidia_ctl_cdev);
    }

    return 0;
}

==> NVIDIA-FreeBSD-x86-100.14.19.tar.gz: NVIDIA-FreeBSD-x86-100.14.19/src/nvidia_ctl.c <==
/* _NVRM_COPYRIGHT_BEGIN_
 *
 * Copyright 2001-2002 by NVIDIA Corporation.  All rights reserved.  All
 * information contained herein is proprietary and confidential to NVIDIA
 * Corporation.  Any use, reproduction, or disclosure without the written
 * permission of NVIDIA Corporation is prohibited.
 *
 * _NVRM_COPYRIGHT_END_
 */

#include "nv-misc.h"
#include "os-interface.h"
#include "nv.h"
#include "nv-freebsd.h"

static d_open_t  nvidia_ctl_open;
static d_close_t nvidia_ctl_close;
static d_ioctl_t nvidia_ctl_ioctl;
static d_poll_t  nvidia_ctl_poll;

static struct cdevsw nvidia_ctl_cdevsw = {
    .d_open =      nvidia_ctl_open,
    .d_close =     nvidia_ctl_close,
    .d_ioctl =     nvidia_ctl_ioctl,
    .d_poll =      nvidia_ctl_poll,
    .d_name =      "nvidiactl",
    .d_version =   D_VERSION,
    .d_flags =     D_TRACKCLOSE|D_NEEDGIANT
};

static struct cdev *nvidia_ctl_cdev = NULL;
struct nvidia_softc nvidia_ctl_sc;

static int nvidia_count = 0;

int nvidia_ctl_open(
    struct cdev *dev,
    int oflags,
    int devtype,
    d_thread_t *td
)
{
    int status;
    nv_state_t *nv = &nvidia_ctl_state;

    nv_lock_api(nv);
    status = nvidia_open_ctl(dev, td);
    nv_unlock_api(nv);

    return status;
}

int nvidia_ctl_close(
    struct cdev *dev,
    int fflag,
    int devtype,
    d_thread_t *td
)
{
    int status;
    nv_state_t *nv = &nvidia_ctl_state;

    nv_lock_api(nv);
    status = nvidia_close_ctl(dev, td);
    nv_unlock_api(nv);

    return status;
}

int nvidia_ctl_ioctl(
    struct cdev *dev,
    u_long cmd,
    caddr_t data,
    int fflag,
    d_thread_t *td
)
{
    int status = 0;
    nv_state_t *nv = &nvidia_ctl_state;

    if (__NV_IOC_TYPE(cmd) != NV_IOCTL_MAGIC)
        return ENOTTY;

    nv_lock_api(nv);

    switch (__NV_IOC_NR(cmd)) {
        case NV_ESC_CARD_INFO:
            do {
                struct nv_ioctl_card_info *ci = (void *) data;
                status = nvidia_get_card_info(ci);
            } while (0);
            break;

        case NV_ESC_POST_VBIOS:
            do {
                struct nv_ioctl_post_vbios *pc = (void *) data;
                status = nvidia_post_vbios(pc);
            } while (0);
            break;

        case NV_ESC_CHECK_VERSION_STR:
            do {
                nv_ioctl_rm_api_version_t *params = (void *) data;
                status = (rm_perform_version_check(NULL, params) == RM_OK) ? 0 : -EINVAL;
            } while (0);
            break;

        default:
            status = nvidia_handle_ioctl(dev, cmd, data, fflag, td);
    }

    nv_unlock_api(nv);
    return status;
}

int nvidia_ctl_poll(
    struct cdev *dev,
    int events,
    d_thread_t *td
)
{
    nv_state_t *nv;
    struct nvidia_softc *sc;
    struct nvidia_event *et;

    nv = &nvidia_ctl_state;
    sc = nv->os_state;

    nv_lock_rm(nv);

    STAILQ_FOREACH(et, &sc->event_queue, queue) {
        if (et->event.file == __TD_FDT(td))
            break;
    }

    if (et == NULL) {
        nv_unlock_rm(nv);
        selrecord(td, &sc->rsel);
    } else {
        nv_unlock_rm(nv);
        return (events & (POLLIN | POLLPRI | POLLRDNORM));
    }

    return 0;
}

int nvidia_ctl_attach(void)
{
    if (nvidia_count == 0) {
        /*
         * This routine is called from nvidia_attach(), multiple times
         * when more than one device is installed.
         */
        nvidia_ctl_cdev = make_dev(&nvidia_ctl_cdevsw,
                CDEV_CTL_MINOR,
                UID_ROOT, GID_WHEEL, 0666,
                "%s", nvidia_ctl_cdevsw.d_name);
    }

    nvidia_count++;
    return 0;
}

int nvidia_ctl_detach(void)
{
    struct nvidia_softc *sc;

    sc = &nvidia_ctl_sc;
    nvidia_count--;

    if (nvidia_count == 0) {
        /*
         * Like nvidia_ctl_attach(), nvidia_ctl_detach() will also be
         * called more than once with multiple devices.
         */
        destroy_dev(nvidia_ctl_cdev);
    }

    return 0;
}
